# coding: utf-8
from datetime import date

from django.apps import apps
from django.db import router
from django.test.testcases import TestCase

from django_postgres_partitioning import drop_partitions_before_date
from django_postgres_partitioning import get_model_partitions
from django_postgres_partitioning import init
from django_postgres_partitioning import is_initialized
from django_postgres_partitioning import is_model_partitioned
from django_postgres_partitioning import set_partitioning_for_model


class PartitioningTestCase(TestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.model_cls = apps.get_model('core', 'Person')
        cls.db_alias = router.db_for_write(cls.model_cls)

    def _create_objects(self):
        self.person1 = self.model_cls.objects.create(
            last_name='Иванов',
            first_name='Иван',
            middle_name='Иванович',
            created=date(2000, 1, 1),
        )
        self.person2 = self.model_cls.objects.create(
            last_name='Петров',
            first_name='Пётр',
            middle_name='Петрович',
            created=date(2000, 2, 1),
        )

    def test(self):
        self.assertFalse(is_initialized(self.db_alias))

        init(self.db_alias, force=True)

        self.assertTrue(is_initialized(self.db_alias))

        self.assertFalse(is_model_partitioned(self.model_cls))

        set_partitioning_for_model(self.model_cls, 'created', force=True)

        self.assertTrue(is_model_partitioned(self.model_cls))

        self._create_objects()

        self.assertCountEqual(
            (self.person1, self.person2),
            self.model_cls.objects.all()
        )

        partitions = get_model_partitions(self.model_cls)
        self.assertCountEqual(
            ('core_person_y2000m01', 'core_person_y2000m02'),
            partitions
        )

        drop_partitions_before_date(self.model_cls, date(2000, 1, 31))
        self.assertCountEqual(
            ('core_person_y2000m02',),
            get_model_partitions(self.model_cls)
        )
